var $ = require('./utils');

var glob = require('glob');

var chokidar;

function sync(id, src, cache) {
  var entry = cache.get(id) || {};

  if (!$.exists(id)) {
    entry.deleted = true;
  }

  entry.dirty = true;

  if (entry.id) {
    cache.each(function(dep, _id) {
      if (!dep.id) {
        return;
      }

      if (dep.dest === entry.dest) {
        dep.dirty = true;

        if (src.indexOf(_id) === -1) {
          src.push(_id);
        }
      }
    });
  }

  if (entry.deps) {
    entry.deps.forEach(function(dep) {
      var _entry = cache.get(dep);

      if (_entry && _entry.main) {
        _entry.dirty = true;

        if (src.indexOf(dep) === -1) {
          src.push(dep);
        }
      }
    });
  }
}

function watch(cb) {
  chokidar = chokidar || require('chokidar');

  var src = [];

  var timeout,
      cache = this.cache,
      options = this.opts;

  function next(err) {
    cb(err, src);
    src = [];
  }

  function add(file) {
    if (src.indexOf(file) === -1) {
      src.push(file);
      sync(file, src, cache);
    }

    clearTimeout(timeout);
    timeout = setTimeout(next.bind(this), options.interval || 200);
  }

  var watcher = chokidar.watch(options.src, {
    cwd: options.cwd,
    ignored: options.ignore,
    persistent: true,
    ignoreInitial: true,
    followSymlinks: options.followSymlinks
  }).on('all', function(evt, file) {
    switch (evt) {
      case 'add':
      case 'change':
      case 'unlink':
        add.call(this, file);
      break;
    }
  }).on('error', function(e) {
    next.call(this, e);
  });

  if (options.watch && options.watch !== true) {
    watcher.add(options.watch);
  }
}

module.exports = function(cb) {
  var self = this;

  try {
    var files = glob.sync(this.opts.src, {
      dot: true,
      nodir: true,
      nosort: true,
      ignore: this.opts.ignore,
      cwd: this.opts.cwd
    });
  } catch (e) {
    return cb(e);
  }

  if (self.opts.watch) {
    watch.call(self, cb);
  }

  files.forEach(function(file) {
    var entry = self.cache.get(file);

    if (!entry) {
      return;
    }

    if (!$.exists(file)) {
      entry.deleted = true;
      return;
    }

    if (entry.mtime && ($.mtime(file) > entry.mtime)) {
      sync(file, files, self.cache);
    }

    entry.dirty = (self.opts.force === true)
      || (entry.deleted || entry.dirty || !entry.mtime);
  });

  cb(undefined, files);
};
